package cn.com.qmth.stmms.biz.user.service.impl;

import cn.com.qmth.stmms.biz.user.dao.UserDao;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.user.service.query.UserSearchQuery;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.UserSource;
import cn.com.qmth.stmms.common.utils.EncryptUtils;

import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * 用户Service
 *
 * @author LS
 */
@Service("userService")
public class UserServiceImpl implements UserService {

    @Autowired
    private UserDao userDao;

    @Override
    @Transactional
    public User save(User user) {
        if (user.getId() != null && user.getId() > 0) {
            user.setUpdatedTime(new Date());
        } else {
            user.setCreatedTime(new Date());
        }
        return userDao.save(user);
    }

    @Override
    public User findByLoginName(String loginName) {
        return userDao.findFirstByLoginName(loginName);
    }

    @Override
    public User findByMarkerId(Integer markerId) {
        return userDao.findByMarkerId(markerId);
    }

    @Override
    public User findById(Integer id) {
        return userDao.findOne(id);
    }

    @Override
    public User findSchoolAdmin(int schoolId) {
        UserSearchQuery query = new UserSearchQuery();
        query.setSchoolId(schoolId);
        query.addIncludeRole(Role.SCHOOL_ADMIN);
        query.setPageNumber(1);
        query.setPageSize(1);
        query = findByQuery(query);
        if (query.getCurrentCount() > 0) {
            return query.getResult().get(0);
        } else {
            return null;
        }
    }

    @Override
    public User findSchoolViewer(Integer schoolId) {
        UserSearchQuery query = new UserSearchQuery();
        query.setSchoolId(schoolId);
        query.addIncludeRole(Role.SCHOOL_VIEWER);
        query.setPageNumber(1);
        query.setPageSize(1);
        query = findByQuery(query);
        if (query.getCurrentCount() > 0) {
            return query.getResult().get(0);
        } else {
            return null;
        }
    }

    @Override
    public UserSearchQuery findByQuery(final UserSearchQuery query) {
        if (query.getPageNumber() < 1) {
            query.setPageNumber(1);
        }
        if (query.getPageSize() < 1) {
            query.setPageSize(10);
        }

        Page<User> result = userDao.findAll(new Specification<User>() {

            @Override
            public Predicate toPredicate(Root<User> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getSchoolId() > 0) {
                    predicates.add(cb.equal(root.get("schoolId"), query.getSchoolId()));
                }
                if (StringUtils.isNotBlank(query.getLoginName())) {
                    predicates.add(cb.like(root.get("loginName").as(String.class), query.getLoginName() + "%"));
                }
                if (StringUtils.isNotBlank(query.getName())) {
                    predicates.add(cb.like(root.get("name").as(String.class), query.getName() + "%"));
                }
                if (query.getEnable() != null) {
                    predicates.add(cb.equal(root.get("enable"), query.getEnable()));
                }
                if (query.getRole() != null) {
                    predicates.add(cb.equal(root.get("role"), query.getRole()));
                }
                if (query.getSource() != null) {
                    predicates.add(cb.equal(root.get("source"), query.getSource()));
                }
                if (query.getRelatedAccount() != null) {
                    predicates.add(cb.equal(root.get("relatedAccount"), query.getRelatedAccount()));
                }
                if (query.getIncludeRoles() != null && query.getIncludeRoles().size() > 0) {
                    for (Role role : query.getIncludeRoles()) {
                        predicates.add(cb.equal(root.get("role"), role));
                    }
                }
                return predicates.isEmpty() ? cb.conjunction() : cb.and(predicates.toArray(new Predicate[predicates
                        .size()]));
            }
        }, query);
        if (result != null) {
            query.setTotalCount(result.getTotalElements());
            query.setTotalPage(result.getTotalPages());
            query.setResult(result.getContent());
            query.setCurrentCount(result.getNumberOfElements());
        } else {
            query.setTotalCount(0);
            query.setTotalPage(0);
            query.setCurrentCount(0);
        }
        return query;
    }

    @Override
    @Transactional
    public User login(String loginName, String password, String ip) {
        User user = findByLoginName(loginName);
        if (user != null && user.getPassword().equals(EncryptUtils.md5(password))) {
            user.setLastLoginIp(StringUtils.trimToEmpty(ip));
            user.setLastLoginTime(new Date());
            user.refreshAccessToken();
            return userDao.save(user);
        }
        return null;
    }

    @Override
    @Transactional
    public User externalLogin(Integer schoolId, String account, String name, String password, Role role) {
        User user = externalSaveAndUpdate(schoolId, account, name, password, role);
        user.setLastLoginTime(new Date());
        user.refreshAccessToken();
        return userDao.save(user);
    }

    @Override
    public User externalSaveAndUpdate(Integer schoolId, String account, String name, String password, Role role) {
        User user = userDao.findFirstBySchoolIdAndRelatedAccount(schoolId, account);
        if (user == null) {
            user = new User();
            user.setLoginName(schoolId + "-X-" + account);
            user.setName(account);
            user.setPassword(RandomStringUtils.random(16, true, true));
            user.setRole(role);
            user.setSource(UserSource.EXTERNAL);
            user.setEnable(true);
            user.setSchoolId(schoolId);
            user.setRelatedAccount(account);
            user.setCreatedTime(new Date());
            user.setUpdatedTime(new Date());
        }
        if (StringUtils.isNotBlank(name)) {
            user.setName(name);
        }
        if (StringUtils.isNotBlank(password)) {
            user.setPassword(EncryptUtils.md5(password));
        }
        return userDao.save(user);
    }

}
